-------------------------------------------------------------------------------
-- LISTENER by Tammya-MoonGuard (2016)
-------------------------------------------------------------------------------

local Main = ListenerAddon
local L = Main.Locale
local SharedMedia = LibStub("LibSharedMedia-3.0")
 
-- ordered list of chat history entries that should appear in the frame
Main.chat_buffer = {}
local g_chat_buffer_size = 300
local g_old_time = 60*10 -- time until messages are old

local g_show_say = true
local g_show_party = true 
local g_has_unread_entries = false

local g_clickblock = 0

local function MixCol( a, b )
	
	local c = {
		a[1] + b[1];
		a[2] + b[2];
		a[3] + b[3];
	}
	
	local m = math.max( c[1], c[2] )
	m = math.max( m, c[3] )
	if m > 1.0 then
		c[1] = c[1] / m
		c[2] = c[2] / m
		c[3] = c[3] / m
	end
	
	c[1] = math.max( c[1], 0 )
	c[2] = math.max( c[2], 0 )
	c[3] = math.max( c[3], 0 )
	
	return c
end

local g_chat_colors = {
	SAY            = { 0.85,   0.85,   0.85 };
	EMOTE          = { 0.9, 0.45, 0.1 };
	TEXT_EMOTE     = { 0.9, 0.45, 0.1 };
	WHISPER        = { 1,   0,    1 };
	WHISPER_INFORM = { 1,   0,    1 };
	PARTY          = { 0.4, 0.7,  0.9 };
	PARTY_LEADER   = { 0.4, 0.7,  0.9 } ;
	RAID           = { 0.4, 0.7,  0.9 };
	RAID_LEADER    = { 0.4, 0.7,  0.9 } ;
	RAID_WARNING   = { 1.0, 0.5,  0.05 };
	YELL           = { 1.0, 0.0,  0.0 };
}

local g_pcolor   = {-0.25,1,0}
local g_newcolor = {1,1,-0.5}
--[[
local g_chat_colors_player = {
	"SAY"            = MixCol( g_chat_colors["SAY"],            g_pcolor );
	"EMOTE"          = MixCol( g_chat_colors["EMOTE"],          g_pcolor );
	"TEXT_EMOTE"     = MixCol( g_chat_colors["TEXT_EMOTE"],     g_pcolor );
	"WHISPER"        = MixCol( g_chat_colors["WHISPER"],        g_pcolor );
	"WHISPER_INFORM" = MixCol( g_chat_colors["WHISPER_INFORM"], g_pcolor );
	"PARTY" 		 = MixCol( g_chat_colors["PARTY"],          g_pcolor );
	"PARTY_LEADER"   = MixCol( g_chat_colors["PARTY_LEADER"],   g_pcolor );
	"RAID"           = MixCol( g_chat_colors["RAID"],           g_pcolor );
	"RAID_LEADER"    = MixCol( g_chat_colors["RAID_LEADER"],    g_pcolor );
	"RAID_WARNING"   = MixCol( g_chat_colors["RAID_WARNING"],   g_pcolor );
	"YELL"           = MixCol( g_chat_colors["YELL"],           g_pcolor );
	
}]]	

function Main:HasUnreadEntries()
	return g_has_unread_entries
end
--[[
-------------------------------------------------------------------------------
function Main:SetupFrame()
	self.frame = CreateFrame( "Frame", "ListenerFrame", UIParent )
	self.frame:SetSize( 300, 200 )
	self.frame:SetPoint( "CENTER", 0, 0 )
	
	self.scrollmark = CreateFrame( "Frame", nil, self.frame )
	self.scrollmark:SetPoint( "TOPLEFT", self.frame, "BOTTOMLEFT", 0, 4 )
	self.scrollmark:SetPoint( "BOTTOMRIGHT", self.frame, "BOTTOMRIGHT", 0, 0 )
	self.scrollmark.tex = self.scrollmark:CreateTexture()
	self.scrollmark.tex:SetAllPoints()
	self.scrollmark.tex:SetTexture( 1, 0.21, 0.04, 1 )
	self.scrollmark:Hide()
	
	self.scrollmark:SetScript( "OnUpdate", function()
		if g_has_unread_entries then
			-- flash red if we have new entries
			local a = math.sin(GetTime() * 3) * 0.4 + 0.6
			self.scrollmark.tex:SetTexture( 1, 0.21, 0.04, a )
		else
			self.scrollmark.tex:SetTexture( 1, 1, 1, 0.25 )
		end
		
	end)
	
	self.frame.resize = CreateFrame( "Frame", nil, self.frame )
	self.frame.resize:SetPoint( "BOTTOMRIGHT", 0, 0 )
	self.frame.resize:SetSize( 16, 16 )
	
	
	self.frame.bar = CreateFrame( "Frame", "ListenerFrameBar", ListenerFrame )
	self.frame.bar.tex = self.frame.bar:CreateTexture()
	self.frame.bar:SetPoint( "TOPLEFT", 0, 0 )
	self.frame.bar:SetPoint( "BOTTOMRIGHT", self.frame, "TOPRIGHT", 0, -7 ) 
	self.frame.bar.tex:SetTexture( 1,1,1,0.25 )
	self.frame.bar.tex:SetAllPoints()
	
	self.frame.bg = self.frame:CreateTexture( nil, "BACKGROUND" )
	self.frame.bg:SetAllPoints()
	self.frame.bg:SetTexture( 0,0,0,0.25 )
	
	self.chatbox = CreateFrame( "ScrollingMessageFrame", "ListenerChat", ListenerFrame )
	self.chatbox:SetPoint( "TOPLEFT", 2, -16 )
	self.chatbox:SetPoint( "BOTTOMRIGHT", -2, 2 )
	self.chatbox:SetMaxLines( 500 )
	self.chatbox:SetTimeVisible( 300 )
	 
	self.chatbox:SetFont( "Fonts\\ARIALN.TTF", 12, "OUTLINE" )
	self.chatbox:SetJustifyH( "LEFT" )
	self.chatbox:EnableMouseWheel( true )
	--self.chatbox:EnableMouse( true )
	self.chatbox:SetScript("OnMouseWheel", function(self, delta)
		
		local reps = IsShiftKeyDown() and 5 or 1
		if delta > 0 then
			if IsAltKeyDown() then
				self:ScrollToTop()
			elseif IsControlKeyDown() then
				-- todo: increase font size
				Main:SetFontSize( Main.db.profile.font.size + 1 )
			else
				for i = 1,reps do self:ScrollUp() end
			end
		else
			if IsAltKeyDown() then
				self:ScrollToBottom()
			elseif IsControlKeyDown() then
				-- todo: decrease font size
				Main:SetFontSize( Main.db.profile.font.size - 1 )
			else
				for i = 1,reps do self:ScrollDown() end
			end
		end
	end)
	
	self.chatbox:SetScript( "OnMessageScrollChanged", function( self ) 
		-- show or hide the scroll marker if we are scrolled up
		if self:GetCurrentScroll() ~= 0 then
			Main.scrollmark:Show()
		else
			Main.scrollmark:Hide()
		end
	end)
	
	self.chatbox:SetScript( "OnHyperlinkClick", function(self, link, text, button)
	
		if ( strsub(link, 1, 6) == "player" ) then
			local namelink, isGMLink;
			if ( strsub(link, 7, 8) == "GM" ) then
				namelink = strsub(link, 10);
				isGMLink = true;
			else
				namelink = strsub(link, 8);
			end
			
			local name, lineid, chatType, chatTarget = strsplit(":", namelink);
			if IsShiftKeyDown() and button == "RightButton" then
				Main:RemovePlayer( name )
				return
			end
		end
		
		SetItemRef(link, text, button, self);
	
	end)
	
	self.frame.bar:EnableMouse( true )
	self.frame:SetMovable( true )
	self.frame:SetResizable( true )
	self.frame:SetMinResize( 200, 100 )
	
	self.frame.bar:SetScript( "OnMouseDown", 
		function( self, button )
			if button == "LeftButton" then
				Main.frame:StartMoving() 
			end
		end)
		
	self.frame.bar:SetScript( "OnMouseUp", 
		function( self, button )
			if button == "LeftButton" then
				Main.frame:StopMovingOrSizing()
				Main:ClipFramePosition()
				Main:SaveFramePosition()
			end
		end)
	
	self.frame.resize:EnableMouse()
	self.frame.resize:SetScript( "OnMouseDown", 
		function( self, button ) 
			if button == "LeftButton" then
				Main.frame:StartSizing()
			end
		end)
		
	self.frame.resize:SetScript( "OnMouseUp", 
		function( self, button ) 
			if button == "LeftButton" then
				Main.frame:StopMovingOrSizing()
				Main:ClipFramePosition()
				Main:SaveFramePosition()
			end
		end)
		
	-- BUTTONS
	local e
	--[-[
	e = CreateFrame( "Button", nil, self.frame )
	self.button_add = e
	e:SetPoint( "CENTER", self.frame, "TOPLEFT", 0, 0 )
	e:SetSize( 32, 32 )
	e:SetNormalTexture( "Interface\\Minimap\\UI-Minimap-ZoomInButton-Up" )
	e:SetPushedTexture( "Interface\\Minimap\\UI-Minimap-ZoomInButton-Down" )
	e:SetDisabledTexture( "Interface\\Minimap\\UI-Minimap-ZoomInButton-Disabled" )
	e:SetHighlightTexture( "Interface\\Minimap\\UI-Minimap-ZoomButton-Highlight", "ADD" )
	e:SetHitRectInsets( 4, 4, 2, 6 )]-]
	
	e = CreateFrame( "Button", nil, self.frame.bar )
	self.toggle_player = e
	e:SetPoint( "CENTER", self.frame, "TOPLEFT", 12, 0 )
	e:SetSize( 16, 16 )
	e.tex = e:CreateTexture()
	e.tex:SetAllPoints() 
	e.tex:SetTexture( 0.5,1,0.5,0.2 )
	e:SetScript( "OnClick", function() Main:TogglePlayerClicked() end )
	
	e = CreateFrame( "Button", nil, self.frame.bar )
	self.toggle_say = e
	e:SetPoint( "CENTER", self.frame, "TOPLEFT", 32, 0 )
	e:SetSize( 16, 16 )
	e.tex = e:CreateTexture()
	e.tex:SetAllPoints() 
	e.tex:SetTexture( 1,1,1,0.8 )
	e:SetScript( "OnClick", function() Main:ToggleSay() end )
	
	e = CreateFrame( "Button", nil, self.frame.bar )
	self.toggle_party = e
	e:SetPoint( "CENTER", self.frame, "TOPLEFT", 52, 0 )
	e:SetSize( 16, 16 )
	e.tex = e:CreateTexture()
	e.tex:SetAllPoints() 
	e.tex:SetTexture( 0.05,0.2,1,0.8 )
	e:SetScript( "OnClick", function() Main:ToggleParty() end )
	
	e = CreateFrame( "Button", nil, self.frame.bar )
	self.button_read = e
	e:SetPoint( "CENTER", self.frame, "TOPLEFT", 72, 0 )
	e:SetSize( 16, 16 )
	e.tex = e:CreateTexture()
	e.tex:SetAllPoints() 
	e.tex:SetTexture( 0.95,0.90,0.13,0.2 )
	e:SetScript( "OnClick", function() Main:MarkAllRead() end )
end
]]


function Main:SetClickBlock()
	g_clickblock = GetTime()
end

function Main:OnChatboxScroll( delta )
	
	local reps = IsShiftKeyDown() and 5 or 1
	if delta > 0 then
		if IsAltKeyDown() then
			ListenerFrameChat:ScrollToTop()
		elseif IsControlKeyDown() then
			-- todo: increase font size
			Main:SetFontSize( Main.db.profile.frame.font.size + 1 )
		else
			for i = 1,reps do ListenerFrameChat:ScrollUp() end
		end
	else
		if IsAltKeyDown() then
			ListenerFrameChat:ScrollToBottom()
		elseif IsControlKeyDown() then
			-- todo: decrease font size
			Main:SetFontSize( Main.db.profile.frame.font.size - 1 )
		else
			for i = 1,reps do ListenerFrameChat:ScrollDown() end
		end
	end
end

function Main:OnChatboxHyperlinkClick( link, text, button )
	if GetTime() - g_clickblock < 0.4 then
		-- block clicks when scroll changes
		return
	end	
	
	if button == "LeftButton" and not IsShiftKeyDown() then
		return
	end
	
	if ( strsub(link, 1, 6) == "player" ) then
		local namelink, isGMLink;
		if ( strsub(link, 7, 8) == "GM" ) then
			namelink = strsub(link, 10);
			isGMLink = true;
		else
			namelink = strsub(link, 8);
		end
		
		local name, lineid, chatType, chatTarget = strsplit(":", namelink);
		if IsShiftKeyDown() and button == "RightButton" then
			Main:RemovePlayer( name )
			return
		end
	end
	
	SetItemRef(link, text, button, self);

end

-------------------------------------------------------------------------------
function Main:FlagUnreadEntries()
	ListenerFrameBarRead.tex:SetColorTexture( 0.95,0.90,0.13,0.8 )
	g_has_unread_entries = true
end

-------------------------------------------------------------------------------
function Main:UnflagUnreadEntries()
	ListenerFrameBarRead.tex:SetColorTexture( 0.95,0.90,0.13,0.2 )
	g_has_unread_entries = false
end

-------------------------------------------------------------------------------
function Main:TogglePlayerClicked()
	if UnitExists( "target" ) then 
		if UnitIsPlayer( "target" ) and UnitIsFriend( "target", "player" ) then
			if UnitGUID("target") == UnitGUID( "player" ) then return end
			
			self:TogglePlayer( UnitName( "target" ), true )
		end
	end
end

-------------------------------------------------------------------------------
function Main:ProbePlayer()
	local unit
	if UnitExists( "target" ) then 
		unit = "target"
	else
		unit = "mouseover"
	end
	
	local on = false
	
	if UnitGUID( unit ) == UnitGUID( 'player' ) then
		on = true
	elseif UnitIsPlayer( unit ) and UnitIsFriend( unit, "player" ) then
		local name = UnitName( unit )
		
		if name then
			on = self.player_list[name]
		end
	end
	
	if on then 
		ListenerFrameBarToggle.tex:SetColorTexture( 0.5,1,0.5,0.8 )
	else
		ListenerFrameBarToggle.tex:SetColorTexture( 0.5,1,0.5,0.2 )
	end
end

-------------------------------------------------------------------------------
function Main:ToggleSay()
	g_show_say = not g_show_say
	if g_show_say then
		ListenerFrameBarShowSay.tex:SetColorTexture( 1,1,1,0.8 )
	else
		ListenerFrameBarShowSay.tex:SetColorTexture( 1,1,1,0.25 )
	end
	Main:RefreshChat()
end

-------------------------------------------------------------------------------
function Main:ToggleParty()
	g_show_party = not g_show_party
	if g_show_party then
		ListenerFrameBarShowParty.tex:SetColorTexture( 0.05,0.2,1,0.8 )
	else
		ListenerFrameBarShowParty.tex:SetColorTexture( 0.05,0.2,1,0.25 )
	end
	Main:RefreshChat()
end

-------------------------------------------------------------------------------
function Main:ClipFramePosition()
	-- todo
end
 
-------------------------------------------------------------------------------
function Main:LoadFrameSettings()

	local left = self.db.profile.frame.x or (GetScreenWidth()/2)
	local bottom  = self.db.profile.frame.y or (GetScreenHeight()/2)
	
	left   = math.max( left, 0 )
	bottom = math.max( bottom, 0 )
	left   = math.min( left, GetScreenWidth() - 32 )
	bottom = math.min( bottom, GetScreenHeight() - 32 )
	
	if self.db.profile.frame.width < 50 then
		self.db.profile.frame.width = 50
	end
	
	if self.db.profile.frame.height < 50 then
		self.db.profile.frame.height = 50
	end
		
	ListenerFrame:SetPoint( "BOTTOMLEFT", left, bottom )
	ListenerFrame:SetSize( self.db.profile.frame.width, self.db.profile.frame.height )
	
	ListenerFrame:SetBackdropColor( self.db.profile.frame.bg.r,
	                                self.db.profile.frame.bg.g,
                                    self.db.profile.frame.bg.b,
	                                self.db.profile.frame.bg.a )
									
	ListenerFrame:SetBackdropBorderColor( self.db.profile.frame.edge.r,
	                                      self.db.profile.frame.edge.g,
                                          self.db.profile.frame.edge.b,
	                                      self.db.profile.frame.edge.a )
	
	self:LoadChatFont()
	
	if self.db.profile.frame.hidden then
		ListenerFrame:Hide()
	else
		ListenerFrame:Show()
	end
end

-------------------------------------------------------------------------------
function Main:ToggleFrame()
	self.db.profile.frame.hidden = not self.db.profile.frame.hidden
	if self.db.profile.frame.hidden then
		ListenerFrame:Hide()
	else
		ListenerFrame:Show()
	end
end

-------------------------------------------------------------------------------
function Main:HideFrame( dontsave )
	ListenerFrame:Hide()
	if not dontsave then
		self.db.profile.frame.hidden = true
	end
end

-------------------------------------------------------------------------------
function Main:ShowFrame( dontsave )
	ListenerFrame:Show()
	if not dontsave then
		self.db.profile.frame.hidden = false
	end
end

-------------------------------------------------------------------------------
function Main:SaveFramePosition()
	self.db.profile.frame.x = ListenerFrame:GetLeft()
	self.db.profile.frame.y = ListenerFrame:GetBottom()
	self.db.profile.frame.width = ListenerFrame:GetWidth()
	self.db.profile.frame.height = ListenerFrame:GetHeight()
end

-------------------------------------------------------------------------------
local SAY_EVENTS = { SAY=true, EMOTE=true, TEXT_EMOTE=true, YELL=true }

-------------------------------------------------------------------------------
local function EntryFilter( entry )
	if SAY_EVENTS[entry.e] then
		return g_show_say
	elseif g_show_party then
		return true
	end
end

local function GetEntryColor( e )
	
	local color = g_chat_colors[ e.e ]
	-- WARNING: this is a REFERENCE
	
	if not e.r and not e.p then
		-- unread message
		color = MixCol( color, g_newcolor )
	end
	
	if e.p then
		-- player message
		color = MixCol( color, g_pcolor )
	end
	
	if GetTime() > e.t + 600 then
		-- todo: old time
		color = MixCol( color, { -0.5, -0.5, -0.5 } )
	elseif GetTime() > e.t + 300 then
		color = MixCol( color, { -0.25, -0.25, -0.25 } )
	end
	
	return color
end

local g_msgtypes = { 
	SAY=1, PARTY=1, PARTY_LEADER=1, RAID=1, RAID_LEADER=1, RAID_WARNING=1, YELL=1;
	EMOTE=2;
	TEXT_EMOTE=3;
}	

local g_m1_prefix = {
	PARTY        = "[P] ";
	PARTY_LEADER = "[PL] ";
	RAID         = "[R] ";
	RAID_LEADER  = "[RL] ";
	RAID_WARNING = "[RW] ";
}

function Main:FormatChatMessage( e )
	local msgtype = g_msgtypes[e.e]
	
	-- get icon and name
	local name, _, icon, color --
	
	local stamp = ""
	if Main.db.profile.frame.timestamps then
		stamp = date( "[%H:%M] ", e.t )
		stamp = "|cff808080" .. stamp .. "|r"
	end
	
	if not e.p then 
		name, _, icon, color = self:GetICName( e.s, e.g )
		
		if icon then
			icon = "|TInterface\\Icons\\" .. icon .. ":0|t "
		else
			icon = ""
		end
		
		if color then
			name = "|c" .. color .. name .. "|r"
		end
		
		name = "|Hplayer:" .. e.s .. "|h" .. name .. "|h" 
	end
	 
	if msgtype == 1 then
	   
		local prefix = g_m1_prefix[e.e] or ""
		prefix = stamp .. prefix
		
		if e.p then
			return string.format( "%s>> %s", prefix, e.m )
		else
			return string.format( "%s%s%s: %s", prefix, icon, name, e.m )
		end
	elseif msgtype == 2 then
		if e.p then
			return string.format( "%s** %s", stamp, e.m )
		else
			return string.format( "%s%s%s %s", stamp, icon, name, e.m )
		end
	elseif msgtype == 3 then
	
		if e.p then 
			return string.format( "%s** %s", stamp, e.m )
		else
			local msg = e.m:gsub( e.s, name )
			return string.format( "%s%s%s", stamp, icon, msg )
		end
	
	end
	return ""
	
end

-------------------------------------------------------------------------------
function Main:AddMessage( e, beep )
	if not EntryFilter( e ) then return end
	
	table.insert( self.chat_buffer, e )
	while #self.chat_buffer > g_chat_buffer_size do
		table.remove( self.chat_buffer, 1 )
	end
	
	if not e.r and not e.p then
		if beep and Main.db.profile.sound.msg and ListenerFrame:IsShown() then
			PlaySoundFile( SharedMedia:Fetch( "sound", "ListenerBeep" ), "Master" )
		end
		self:FlagUnreadEntries()
	end
	
	local color = GetEntryColor( e )
	ListenerFrameChat:AddMessage( self:FormatChatMessage( e ), 
	                              color[1], color[2], color[3], e.id )
end

-------------------------------------------------------------------------------
function Main:UpdateMessageColor( e )
	local color = GetEntryColor( e ) 
	ListenerFrameChat:UpdateColorByID( e.id, color[1], color[2], color[3] )
end

-------------------------------------------------------------------------------
function Main:CheckUnread()
	self:UnflagUnreadEntries()
	g_has_unread_entries = false
	for k,v in pairs( self.unread_entries ) do
		if EntryFilter( v ) and (self.player_list[ v.s ] or guid == UnitGUID("player")) then
			self:FlagUnreadEntries()
		end
	end
end

-------------------------------------------------------------------------------
function Main:RefreshChat()
	ListenerFrameChat:Clear()
	self.chat_buffer = {}
	self:UnflagUnreadEntries()
	local entries = {}
	
	for playername,p in pairs(self.player_list) do
		local history = self.chat_history[playername] or {}
		for k2, v2 in pairs( history ) do
			if EntryFilter( v2 ) then
				table.insert( entries, v2 )
			end
		end
	end
	
	-- our own text
	local history = self.chat_history[UnitName("player")] or {}
	for k2, v2 in pairs( history ) do
		if EntryFilter( v2 ) then
			table.insert( entries, v2 )
		end
	end
	
	-- sort by lineid
	table.sort( entries, function( a, b )
		return a.id < b.id
	end)
	
	-- crop and populate
	local start = #entries - g_chat_buffer_size
	if start < 1 then start = 1 end
	for i = start,#entries do
		self:AddMessage( entries[i] )
	end 
end

-------------------------------------------------------------------------------
function Main:SetChatFont( font )
	self.db.profile.frame.font.face = font
	self:LoadChatFont()
end

-------------------------------------------------------------------------------
function Main:SetChatOutline( value )
	self.db.profile.frame.font.outline = value
	self:LoadChatFont()
end

-------------------------------------------------------------------------------
function Main:SetFontSize( size )
	size = math.max( size, 6 )
	size = math.min( size, 24 )
	self.db.profile.frame.font.size = size
	self:LoadChatFont()
end

function Main:Frame_SetShadow( val )
	self.db.profile.frame.font.shadow = val
	self:LoadChatFont()
end	

-------------------------------------------------------------------------------
function Main:Frame_SetBGColor( r, g, b, a )
	self.db.profile.frame.bg.r = r
	self.db.profile.frame.bg.g = g
	self.db.profile.frame.bg.b = b
	self.db.profile.frame.bg.a = a
	self:LoadFrameSettings()
end

-------------------------------------------------------------------------------
function Main:Frame_SetEdgeColor( r, g, b, a )
	self.db.profile.frame.edge.r = r
	self.db.profile.frame.edge.g = g
	self.db.profile.frame.edge.b = b
	self.db.profile.frame.edge.a = a
	self:LoadFrameSettings()
end

function Main:Frame_SetTimestamps( stamps )
	self.db.profile.frame.timestamps = stamps
	Main:RefreshChat()
end

-------------------------------------------------------------------------------
function Main:LoadChatFont()
	local outline = nil
	if self.db.profile.frame.font.outline == 2 then
		outline = "OUTLINE"
	elseif self.db.profile.frame.font.outline == 3 then
		outline = "THICKOUTLINE"
	end
	local font = SharedMedia:Fetch( "font", self.db.profile.frame.font.face )
	ListenerFrameChat:SetFont( font, self.db.profile.frame.font.size, outline )
	
	if self.db.profile.frame.font.shadow then 
		ListenerFrameChat:SetShadowColor( 0, 0, 0, 0.8 )
		ListenerFrameChat:SetShadowOffset( 1,-1 )
	else
		
		ListenerFrameChat:SetShadowColor( 0, 0, 0, 0 )
	end
end

